#include "Nd2MemoryTest.h"

//Attempt NDConnector::Connect/Accept with the length of private data exceeding the limit of the hardware
//Should get ND_INVALID_BUFFER_SIZE

void Nd2LargePrivateDataServer::RunTest(
    _In_ const struct sockaddr_in& v4Src,
    _In_ DWORD queueDepth,
    _In_ DWORD nSge
    )
{
    //prep
    Nd2TestBase::Init( v4Src);
    Nd2TestBase::CreateMR();
    Nd2TestBase::RegisterDataBuffer(
        x_HdrLen + x_MaxXfer,
        ND_MR_FLAG_ALLOW_LOCAL_WRITE,ND_SUCCESS,
        "Register memory with local read access failed"
        );
    Nd2TestBase::CreateCQ(nSge);
    Nd2TestBase::CreateConnector();
    Nd2TestBase::CreateQueuePair(queueDepth,nSge);
    Nd2TestServerBase::CreateListen();
    Nd2TestServerBase::Listen(v4Src);
    Nd2TestServerBase::GetConnectionRequest();

    //
    // Pre-post receive request.
    //
    Nd2TestBase::PostReceive(NULL,0);

    //get the max supported length of private data on
    ND2_ADAPTER_INFO adaptorInfo;
    adaptorInfo.InfoVersion=ND_VERSION_2;
    ULONG infoSize = sizeof(ND2_ADAPTER_INFO);
    HRESULT hr = m_pAdapter->Query(&adaptorInfo,&infoSize);
    LogIfErrorExit(hr,ND_SUCCESS,"Querying dataptor info failed!",__LINE__);
    DWORD maxCalleeData = adaptorInfo.MaxCalleeData;


    // give it the correct private data size/buffer size, but larger than what the hardware can support
    void* data = new (std::nothrow) char[maxCalleeData+1];
    Nd2TestServerBase::Accept(
        1,
        1,
        data,
        maxCalleeData + 1,
        ND_INVALID_BUFFER_SIZE,
        "Expecting Access violation when data exceeds MaxCalleeData"
        );
    delete[] data;

    //try with just enough data
    data = new (std::nothrow) char[maxCalleeData];
    Nd2TestServerBase::Accept(
        1,
        1,
        data,
        maxCalleeData,
        ND_SUCCESS,
        "Expecting ND_SUCCESS when privata data size is just MaxCalleeData"
        );

    //Get result for the pre-posted receive
    Nd2TestBase::WaitForCompletion(ND_SUCCESS);

    //tear down
    Nd2TestBase::Shutdown();
    delete[] data;
}


void Nd2LargePrivateDataClient::RunTest(
    _In_ const struct sockaddr_in& v4Src,
    _In_ const struct sockaddr_in& v4Dst,
    _In_ DWORD queueDepth,
    _In_ DWORD nSge
    )
{
    //prep
    Nd2TestBase::Init( v4Src);
    Nd2TestBase::CreateMR();
    Nd2TestBase::RegisterDataBuffer(
        x_MaxXfer,
        ND_MR_FLAG_ALLOW_REMOTE_READ | ND_MR_FLAG_ALLOW_REMOTE_WRITE,
        ND_SUCCESS,
        "Register memory with local read access failed"
        );
    Nd2TestBase::CreateCQ(nSge,ND_SUCCESS);
    Nd2TestBase::CreateConnector();
    Nd2TestBase::CreateQueuePair(queueDepth,nSge);

    //get the max supported length of private data
    ND2_ADAPTER_INFO adaptorInfo;
    adaptorInfo.InfoVersion=ND_VERSION_2;
    ULONG infoSize = sizeof(ND2_ADAPTER_INFO);
    HRESULT hr = m_pAdapter->Query(&adaptorInfo,&infoSize);
    LogIfErrorExit(hr,ND_SUCCESS,"Querying dataptor info failed!",__LINE__);
    DWORD maxCallerData = adaptorInfo.MaxCallerData;

    //attempt to connect with larger private data
    const void* data = new (std::nothrow) char[maxCallerData+1];
    Nd2TestClientBase::Connect(
        v4Src,
        v4Dst,
        1,
        1,
        data,
        maxCallerData + 1,
        ND_INVALID_BUFFER_SIZE,
        "Expecting Access violation when data size is bigger than maxCallerData"
        );
    delete[] data;

    //try with just enough data, should get ND_SUCCESS
    data = new (std::nothrow) char[maxCallerData];
    Nd2TestClientBase::Connect(
        v4Src,
        v4Dst,
        1,
        1,
        data,
        maxCallerData,
        ND_SUCCESS,
        "Expecting ND_SUCCESS when data size is just maxCallerData"
        );
    delete[] data;

    //complete connect event
    Nd2TestClientBase::CompleteConnect();

    //send a packet to signal tear down
    Nd2TestBase::Send(NULL,NULL,0,0);
    Nd2TestBase::WaitForCompletion();

    //tear down
    Nd2TestBase::Shutdown();
}
