#include "Nd2MemoryTest.h"

//server attempts to read/write more than the size of its buffer

void Nd2OverReadServer::RunTest(
    _In_ const struct sockaddr_in& v4Src,
    _In_ DWORD /*queueDepth*/,
    _In_ DWORD /*nSge*/
    )
{
    //prep
    Nd2TestBase::Init( v4Src);
    Nd2TestBase::CreateMR();
    Nd2TestBase::RegisterDataBuffer(
        x_HdrLen + x_MaxXfer + 1,
        ND_MR_FLAG_ALLOW_LOCAL_WRITE,
        ND_SUCCESS,
        "Register memory with local write access failed"
        );
    Nd2TestBase::CreateCQ(2);
    Nd2TestBase::CreateConnector();
    Nd2TestBase::CreateQueuePair(1,1);
    Nd2TestServerBase::CreateListen();
    Nd2TestServerBase::Listen(v4Src);
    Nd2TestServerBase::GetConnectionRequest();

    //
    // Pre-post receive request.
    //
    ND2_SGE Sge;
    const ND_MW_DESCRIPTOR* ndmd = reinterpret_cast<const ND_MW_DESCRIPTOR*>(m_Buf);
    Sge.Buffer = m_Buf;
    Sge.BufferLength = sizeof(*ndmd);
    Sge.MemoryRegionToken = m_pMr->GetLocalToken();
    Nd2TestBase::PostReceive(&Sge,1);

    //accept connection
    Nd2TestServerBase::Accept(1,1);

    //Get result for the pre-posted receive
    Nd2TestBase::WaitForCompletion(ND_SUCCESS);
    UINT64 addr = ndmd->Base;
    UINT32 token = ndmd->Token;

    //prepare buffer to read/write

    //read more than the remote peer has
    Sge.BufferLength= x_HdrLen + x_MaxXfer+1;

    //read
    Nd2TestBase::Read(&Sge,1,addr,token,0);

    //wait for read completion
    Nd2TestBase::WaitForCompletion(
        ND_ACCESS_VIOLATION,
        "Read from memory region more its size should get error ND_ACCESS_VIOLATION"
        );

    //tear down
    Nd2TestBase::Shutdown();
}

void Nd2OverWriteServer::RunTest(
    _In_ const struct sockaddr_in& v4Src,
    _In_ DWORD /*queueDepth*/,
    _In_ DWORD /*nSge*/
    )
{

    //prep
    Nd2TestBase::Init( v4Src);
    Nd2TestBase::CreateMR();
    Nd2TestBase::RegisterDataBuffer(
        x_HdrLen + x_MaxXfer + 1,
        ND_MR_FLAG_ALLOW_LOCAL_WRITE,
        ND_SUCCESS,
        "Register memory with local read access failed"
        );
    Nd2TestBase::CreateCQ(2);
    Nd2TestBase::CreateConnector();
    Nd2TestBase::CreateQueuePair(1,1);
    Nd2TestServerBase::CreateListen();
    Nd2TestServerBase::Listen(v4Src);
    Nd2TestServerBase::GetConnectionRequest();

    //
    // Pre-post receive request.
    //
    ND2_SGE Sge;
    const ND_MW_DESCRIPTOR* ndmd = reinterpret_cast<const ND_MW_DESCRIPTOR*>(m_Buf);
    Sge.Buffer = m_Buf;
    Sge.BufferLength = sizeof(*ndmd);
    Sge.MemoryRegionToken = m_pMr->GetLocalToken();
    Nd2TestBase::PostReceive(&Sge,1);

    //accept connection
    Nd2TestServerBase::Accept(1,1);

    //Get result for the pre-posted receive
    Nd2TestBase::WaitForCompletion(ND_SUCCESS);
    UINT64 addr = ndmd->Base;
    UINT32 token = ndmd->Token;

    //prepare buffer to read/write

    //write more than the remote peer has
    Sge.BufferLength= x_HdrLen + x_MaxXfer+1;

    //write
    Nd2TestBase::Write(&Sge,1,addr,token,0);

    //wait for write completion
    Nd2TestBase::WaitForCompletion(
        ND_ACCESS_VIOLATION,
        "Write to memory region when it's being invalidated should result in error"
        );

    Nd2TestBase::Shutdown();
}


void Nd2OverReadWriteClient::RunTest(
    _In_ const struct sockaddr_in& v4Src,
    _In_ const struct sockaddr_in& v4Dst,
    _In_ DWORD /*queueDepth*/,
    _In_ DWORD /*nSge*/
    )
{

    //prep
    Nd2TestBase::Init( v4Src);
    Nd2TestBase::CreateMR();
    Nd2TestBase::RegisterDataBuffer(
        x_MaxXfer,
        ND_MR_FLAG_ALLOW_REMOTE_READ | ND_MR_FLAG_ALLOW_REMOTE_WRITE,
        ND_SUCCESS,
        "Register memory with local read access failed"
        );
    Nd2TestBase::CreateCQ(2,ND_SUCCESS);
    Nd2TestBase::CreateConnector();
    Nd2TestBase::CreateQueuePair(2,1);
    Nd2TestClientBase::Connect(v4Src,v4Dst,1,1);
    Nd2TestClientBase::CompleteConnect();
    Nd2TestBase::CreateMW();
    Nd2TestBase::Bind(x_MaxXfer, ND_OP_FLAG_ALLOW_WRITE | ND_OP_FLAG_ALLOW_READ);

    //prepare memory descriptor
    ND_MW_DESCRIPTOR* ndmd = (ND_MW_DESCRIPTOR*)m_Buf;
    ndmd->Base = (UINT64)m_Buf;
    ndmd->Token = m_pMw->GetRemoteToken();
    ndmd->Length = x_MaxXfer;

    //prepare Sge
    ND2_SGE Sge;
    Sge.Buffer = ndmd;
    Sge.BufferLength=sizeof(*ndmd);
    Sge.MemoryRegionToken = m_pMr->GetLocalToken();

    //send it over
    Nd2TestBase::Send(NULL,&Sge,1,0);

    //Wait for Send completion
    Nd2TestBase::WaitForCompletion();

    //wait 5 seconds
    Sleep(5*1000);

    //tear down
    Nd2TestBase::Shutdown();
}
