#include "Nd2MemoryTest.h"

//Post a send on local QP without posting receive on remote QP. 
//Verify a CQ notification is generated

void Nd2SendNoReceiveServer::RunTest(
    _In_ const struct sockaddr_in& v4Src,
    _In_ DWORD queueDepth,
    _In_ DWORD nSge
    )
{
  
	//prep
	Nd2TestBase::Init( v4Src);
	Nd2TestBase::CreateMR();
	Nd2TestBase::RegisterDataBuffer( x_HdrLen + x_MaxXfer+1,ND_MR_FLAG_ALLOW_LOCAL_WRITE,ND_SUCCESS,"Register memory with local read access failed");
	Nd2TestBase::CreateCQ(nSge);
	Nd2TestBase::CreateConnector();
	Nd2TestBase::CreateQueuePair(queueDepth,nSge);
	Nd2TestServerBase::CreateListen();
	Nd2TestServerBase::Listen(v4Src);
	Nd2TestServerBase::GetConnectionRequest();

	//accept connection
    Nd2TestServerBase::Accept(1,1);
	
	//sleep 5 seconds
	Sleep(5*1000);
	
	//we are not expecting any CQ notification, because a connection is not established
	Nd2TestBase::CheckCQ(0);

	//tear down
	Nd2TestBase::Shutdown();
}


void Nd2SendNoReceiveClient::RunTest(
    _In_ const struct sockaddr_in& v4Src,
    _In_ const struct sockaddr_in& v4Dst,
    _In_ DWORD queueDepth,
    _In_ DWORD nSge
    )
{
    //prep
	Nd2TestBase::Init( v4Src);
	Nd2TestBase::CreateMR();
	Nd2TestBase::RegisterDataBuffer(x_MaxXfer,ND_MR_FLAG_ALLOW_LOCAL_WRITE,ND_SUCCESS,"Register memory with local read access failed");
	Nd2TestBase::CreateCQ(nSge,ND_SUCCESS);
	Nd2TestBase::CreateConnector();
	Nd2TestBase::CreateQueuePair(queueDepth,nSge);
	Nd2TestClientBase::Connect(v4Src,v4Dst,1,1);
	Nd2TestClientBase::CompleteConnect();
	
	//prepare Sge
	ND2_SGE Sge[1];
	Sge[0].Buffer = m_Buf;
	Sge[0].BufferLength=x_MaxXfer;
	Sge[0].MemoryRegionToken = m_pMr->GetLocalToken();

	//send it over
	Nd2TestBase::Send(NULL,Sge,1,0);

	LogInfo("There's no receive posted on the other side, the test case should not hang because we should get a notification in the CQ");
	//expect failure to of the send in the completion queue
	Nd2TestBase::WaitForCompletion(ND_IO_TIMEOUT);
	Nd2TestBase::Shutdown();
}
