#include "Nd2MemoryTest.h"
//attempt to read/write to invalidated MW


void Nd2InvalidReadServer::RunTest(
    _In_ const struct sockaddr_in& v4Src,
    _In_ DWORD /*queueDepth*/,
    _In_ DWORD /*nSge*/
    )
{
    //prep
    Nd2TestBase::Init( v4Src);
    Nd2TestBase::CreateMR();
    Nd2TestBase::RegisterDataBuffer(
        x_HdrLen + x_MaxXfer,
        ND_MR_FLAG_ALLOW_LOCAL_WRITE,
        ND_SUCCESS,
        "Register memory with local read access failed"
        );
    Nd2TestBase::CreateCQ(2);
    Nd2TestBase::CreateConnector();
    Nd2TestBase::CreateQueuePair(1,1);
    Nd2TestServerBase::CreateListen();
    Nd2TestServerBase::Listen(v4Src);
    Nd2TestServerBase::GetConnectionRequest();

    //
    // Pre-post receive request.
    //
    ND2_SGE Sge;
    const ND_MW_DESCRIPTOR* ndmd = reinterpret_cast<const ND_MW_DESCRIPTOR*>(m_Buf);
    Sge.Buffer = m_Buf;
    Sge.BufferLength = sizeof(*ndmd);
    Sge.MemoryRegionToken = m_pMr->GetLocalToken();
    Nd2TestBase::PostReceive(&Sge,1);

    //accept connection
    Nd2TestServerBase::Accept(1,1);

    //Get result for the pre-posted receive
    Nd2TestBase::WaitForCompletion(ND_SUCCESS);
    //get remote address and token
    UINT64 addr = ndmd->Base;
    UINT32 token = ndmd->Token;

    //Wait 5 seconds for memory window to be invalidated
    Sleep(5*1000);

    //try to read from the remote MW, should get error
    Sge.BufferLength= x_HdrLen + x_MaxXfer+1;
    Nd2TestBase::Read(&Sge,1,addr,token,0);
    //wait for read completion, expecting error
    Nd2TestBase::WaitForCompletion(
        ND_ACCESS_VIOLATION,
        "Read from memory window when it's being invalidated should result in error"
        );

    //tear down
    Nd2TestBase::Shutdown();
}


void Nd2InvalidWriteServer::RunTest(
    _In_ const struct sockaddr_in& v4Src,
    _In_ DWORD /*queueDepth*/,
    _In_ DWORD /*nSge*/
    )
{
    //prep
    Nd2TestBase::Init( v4Src);
    Nd2TestBase::CreateMR();
    Nd2TestBase::RegisterDataBuffer(
        x_MaxXfer,
        ND_MR_FLAG_ALLOW_LOCAL_WRITE,
        ND_SUCCESS,
        "Register memory with local read access failed"
        );
    Nd2TestBase::CreateCQ(2);
    Nd2TestBase::CreateConnector();
    Nd2TestBase::CreateQueuePair(1,1);
    Nd2TestServerBase::CreateListen();
    Nd2TestServerBase::Listen(v4Src);
    Nd2TestServerBase::GetConnectionRequest();

    //
    // Pre-post receive request.
    //
    ND2_SGE Sge;
    const ND_MW_DESCRIPTOR* ndmd = reinterpret_cast<const ND_MW_DESCRIPTOR*>(m_Buf);
    Sge.Buffer = m_Buf;
    Sge.BufferLength = sizeof(*ndmd);
    Sge.MemoryRegionToken = m_pMr->GetLocalToken();
    Nd2TestBase::PostReceive(&Sge,1);

    //accept connection
    Nd2TestServerBase::Accept(1,1);

    //Get result for the pre-posted receive
    Nd2TestBase::WaitForCompletion(ND_SUCCESS);

    //get remote address and token
    UINT64 addr = ndmd->Base;
    UINT32 token = ndmd->Token;

    //Wait 5 seconds for memory window to be invalidated
    Sleep(5*1000);

    //try to write to the remote MW, should get error
    Sge.BufferLength= x_HdrLen + x_MaxXfer+1;
    Nd2TestBase::Write(&Sge,1,addr,token,0);
    //wait for write completion,expecting error
    Nd2TestBase::WaitForCompletion(
        ND_ACCESS_VIOLATION,
        "Write to memory window when it's being invalidated should result in error"
        );

    //tear down
    Nd2TestBase::Shutdown();
}


void Nd2InvalidReadWriteClient::RunTest(
    _In_ const struct sockaddr_in& v4Src,
    _In_ const struct sockaddr_in& v4Dst,
    _In_ DWORD /*queueDepth*/,
    _In_ DWORD /*nSge*/
    )
{
    //prep
  Nd2TestBase::Init( v4Src);
    Nd2TestBase::CreateMR();
    Nd2TestBase::RegisterDataBuffer(
        x_HdrLen + x_MaxXfer,
        ND_MR_FLAG_ALLOW_REMOTE_READ | ND_MR_FLAG_ALLOW_REMOTE_WRITE,
        ND_SUCCESS,
        "Register memory with local read access failed"
        );
    Nd2TestBase::CreateCQ(2,ND_SUCCESS);
    Nd2TestBase::CreateConnector();
    Nd2TestBase::CreateQueuePair(2,1);
    Nd2TestClientBase::Connect(v4Src,v4Dst,1,1);
    Nd2TestClientBase::CompleteConnect();
    Nd2TestBase::CreateMW();
    Nd2TestBase::Bind(x_MaxXfer, ND_OP_FLAG_ALLOW_WRITE | ND_OP_FLAG_ALLOW_READ);

    //create memory descriptor
    ND_MW_DESCRIPTOR* ndmd = (ND_MW_DESCRIPTOR*)m_Buf;
    ndmd->Base = (UINT64)m_Buf;
    ndmd->Token = m_pMw->GetRemoteToken();
    ndmd->Length = x_MaxXfer;

    //prepare Sge
    ND2_SGE Sge;
    Sge.Buffer = ndmd;
    Sge.BufferLength=sizeof(*ndmd);
    Sge.MemoryRegionToken = m_pMr->GetLocalToken();

    //post receive to capture connection teardown
    Nd2TestBase::PostReceive(NULL,0);

    //send it over
    Nd2TestBase::Send(NULL,&Sge,1,0);

    //wait for send completion
    Nd2TestBase::WaitForCompletion();

    //invalidate memory window
    Nd2TestBase::InvalidateMW();

    //wait for invalidation completion
    Nd2TestBase::WaitForCompletion();
    
    //wait for receive be flushed
    Nd2TestBase::WaitForCompletion(
        ND_CANCELED,
        "Receive should have been cancelled because the peer is writing to invalid MW"
        );
}
