// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//

#include "Nd2MemoryTest.h"

enum TestName
{
    ND2_INVALIDREAD_TEST,
    ND2_INVALIDWRITE_TEST,
    ND2_LARGEPRIVATEDATA_TEST,
    ND2_MRDEREGISTER_TEST,
    ND2_OVERREAD_TEST,
    ND2_OVERWRITE_TEST,
    ND2_WRITEVIOLATION_TEST,
    ND2_SEND_NORECEIVE_TEST,
    ND2_RECEIVE_FLUSHQP_TEST,
    ND2_SENDRECEIVE_CONN_CLOSED_TEST,
    ND2_LARGEQP_TEST,
    ND2_QPMAX_TEST,
    ND2_MR_INVALIDBUFFER_TEST,
    ND2_CONN_LISTEN_CLOSING_TEST,
    ND2_INVALID_IP_TEST,
    ND2_DUAL_CONECTION_TEST,
    ND2_DUAL_LISTEN_TEST,
    ND2_CONN_REJECT_TEST,
    ND2_CONN_CLOSE_TEST,
    ND2_TEST_ERROR

};
void ShowUsage()
{
    printf( "Nd2MemoryTest t <testname> s|c <ip>[:<port>] [b|p<nSge>] [l<log>]\n"
        "\ts - start as server (listen on IP/Port)\n"
        "\tc - start as client (connect to server IP/Port)\n"
        "\t<ip> - IPv4 Address\n"
        "\t<port> - Port number, default %hu\n"
        "\tl - log output to a file named <log>.\n"
        "\tt - testname"
        "\tChoose from one of the following for test name"
        "\tInvalidRead,InvalidWrite,LargePrivateData,MRDeregister,OverRead,OverWrite,WriteViolation"
        "\tNd2SendNoReceive,Nd2ReceiveFlushQP,Nd2SendReceiveConnClosed,Nd2LargeQPDepth,Nd2QpMax"
        "\tNd2MRInvalidBuffer,Nd2ConnListenClosing,Nd2InvalidIP,Nd2DualListen,Nd2DualConnection,Nd2ConnReject,Nd2ConnClose",
        x_DefaultPort
        );
}

TestName ParseTestName(TCHAR* testName)
{
    if(_tcscmp(testName,_T("Nd2InvalidRead"))==0)
    {
        return ND2_INVALIDREAD_TEST;
    }
    if(_tcscmp(testName,_T("Nd2InvalidWrite"))==0)
    {
        return ND2_INVALIDWRITE_TEST;
    }
    if(_tcscmp(testName,_T("Nd2LargePrivateData"))==0)
    {
        return ND2_LARGEPRIVATEDATA_TEST;
    }
    if(_tcscmp(testName,_T("Nd2MRDeregister"))==0)
    {
        return ND2_MRDEREGISTER_TEST;
    }
    if(_tcscmp(testName,_T("Nd2OverRead"))==0)
    {
        return ND2_OVERREAD_TEST;
    }
    if(_tcscmp(testName,_T("Nd2OverWrite"))==0)
    {
        return ND2_OVERWRITE_TEST;
    }
    if(_tcscmp(testName,_T("Nd2WriteViolation"))==0)
    {
        return ND2_WRITEVIOLATION_TEST;
    }
    if(_tcscmp(testName,_T("Nd2SendNoReceive"))==0)
    {
        return ND2_SEND_NORECEIVE_TEST;
    }
    if(_tcscmp(testName,_T("Nd2ReceiveFlushQP"))==0)
    {
        return ND2_RECEIVE_FLUSHQP_TEST;
    }
    if(_tcscmp(testName,_T("Nd2SendReceiveConnClosed"))==0)
    {
        return ND2_SENDRECEIVE_CONN_CLOSED_TEST;
    }
    if(_tcscmp(testName,_T("Nd2LargeQPDepth"))==0)
    {
        return ND2_LARGEQP_TEST;
    }
    if(_tcscmp(testName,_T("Nd2QpMax"))==0)
    {
        return ND2_QPMAX_TEST;
    }
    if(_tcscmp(testName,_T("Nd2MRInvalidBuffer"))==0)
    {
        return ND2_MR_INVALIDBUFFER_TEST;
    }
    if(_tcscmp(testName,_T("Nd2ConnListenClosing"))==0)
    {
        return ND2_CONN_LISTEN_CLOSING_TEST;
    }
    if(_tcscmp(testName,_T("Nd2InvalidIP"))==0)
    {
        return ND2_INVALID_IP_TEST;
    }
    if(_tcscmp(testName,_T("Nd2DualConnection"))==0)
    {
        return ND2_DUAL_CONECTION_TEST;
    }
    if(_tcscmp(testName,_T("Nd2DualListen"))==0)
    {
        return ND2_DUAL_LISTEN_TEST;
    }

    if(_tcscmp(testName,_T("Nd2ConnReject"))==0)
    {
        return ND2_CONN_REJECT_TEST;
    }
    if(_tcscmp(testName,_T("Nd2ConnClose"))==0)
    {
        return ND2_CONN_CLOSE_TEST;
    }

    return ND2_TEST_ERROR;

}

int __cdecl  _tmain(int argc, TCHAR* argv[])
{
    bool bServer = false;
    bool bClient = false;
    DWORD nSge = 2;
    LONG queueDepth = 64;

    struct sockaddr_in v4Server = {0};
    TestName testName =ND2_INVALIDREAD_TEST;

    WSADATA wsaData;
    int ret = ::WSAStartup( MAKEWORD( 2, 2 ), &wsaData );
    if( ret != 0 )
    {
        printf( "Failed to initialize Windows Sockets: %d\n", ret );
        exit( __LINE__ );
    }

    for( int i = 1; i < argc; i++ )
    {
        const TCHAR* pArg;
        INT len;

        pArg = argv[i];

        // Skip leading dashes/slashes
        while( *pArg == '-' || *pArg == '/' )
        {
            pArg++;
        }

        switch( *pArg )
        {
        case _T('s'):
        case _T('S'):
            bServer = true;
            if( ++i == argc )
            {
                break;
            }

            len = sizeof( v4Server );
            WSAStringToAddress(
                argv[i],
                AF_INET,
                NULL,
                reinterpret_cast<struct sockaddr*>( &v4Server ),
                &len
                );
            break;
        case _T('c'):
        case _T('C'):
            bClient = true;
            if( ++i == argc )
            {
                break;
            }

            len = sizeof( v4Server );
            WSAStringToAddress(
                argv[i],
                AF_INET,
                NULL,
                reinterpret_cast<struct sockaddr*>( &v4Server ),
                &len
                );
            break;
        case _T('l'):
        case _T('L'):
            //
            // Can't use freopen_s because it doesn't allow sharing.
            // So supress the deprecated warning, because for our use
            // it isn't deprecated.
            //
#pragma warning( disable : 4996 )
            if( _tfreopen( ++pArg, _T("w"), stdout ) == NULL ||
                _tfreopen( pArg, _T("a+"), stderr ) == NULL )
#pragma warning( default : 4996 )
            {
                printf( "Could not open log file.\n" );
                exit( __LINE__ );
            }
            break;
        case _T('t'):
        case _T('T'):
            if( ++i == argc )
            {
                break;
            }
            testName = ParseTestName(argv[i]);
            break;
        default:
            _tprintf( _T("Unknown parameter %s\n"), pArg );
            ShowUsage();
            exit( __LINE__ );
        }
    }



    if( (bClient && bServer) || (!bClient && !bServer) )
    {
        printf( "Exactly one of client (c or "
            "server (s) must be specified.\n\n" );
        ShowUsage();
        exit( __LINE__ );
    }

    if( v4Server.sin_addr.s_addr == 0 )
    {
        printf( "Bad address.\n\n");
        ShowUsage();
        exit( __LINE__ );
    }

    if( v4Server.sin_port == 0 )
    {
        v4Server.sin_port = htons( x_DefaultPort );
    }


    OVERLAPPED Ov;
    Ov.hEvent = CreateEvent( NULL, FALSE, FALSE, NULL );
    if( Ov.hEvent == NULL )
    {
        printf( "Failed to allocate event for overlapped operations.\n" );
        exit( __LINE__ );
    }

    HRESULT hr = NdStartup();
    if( FAILED(hr) )
    {
        printf( "NdStartup failed with %08x\n", hr );
        exit( __LINE__ );
    }
    Nd2TestServerBase* server;
    Nd2TestClientBase* client;
    switch(testName)
    {
    case ND2_INVALIDREAD_TEST:
        server = new(std::nothrow) Nd2InvalidReadServer;
        client = new(std::nothrow) Nd2InvalidReadWriteClient;
        break;
    case ND2_INVALIDWRITE_TEST:
        server = new(std::nothrow) Nd2InvalidWriteServer;
        client = new(std::nothrow) Nd2InvalidReadWriteClient;
        break;
    case ND2_LARGEPRIVATEDATA_TEST:
        server = new(std::nothrow) Nd2LargePrivateDataServer;
        client = new(std::nothrow) Nd2LargePrivateDataClient;
        break;
    case ND2_MRDEREGISTER_TEST:
        server = new(std::nothrow) Nd2MRDeregisterServer;
        client = new(std::nothrow) Nd2MRDeregisterClient;
        break;
    case ND2_OVERREAD_TEST:
        server = new(std::nothrow) Nd2OverReadServer;
        client = new(std::nothrow) Nd2OverReadWriteClient;
        break;
    case ND2_OVERWRITE_TEST:
        server = new(std::nothrow) Nd2OverWriteServer;
        client = new(std::nothrow) Nd2OverReadWriteClient;
        break;
    case ND2_WRITEVIOLATION_TEST:
        server = new(std::nothrow) Nd2WriteViolationServer;
        client = new(std::nothrow) Nd2WriteViolationClient;
        break;
    case ND2_SEND_NORECEIVE_TEST:
        server = new(std::nothrow) Nd2SendNoReceiveServer;
        client = new(std::nothrow) Nd2SendNoReceiveClient;
        break;
    case ND2_RECEIVE_FLUSHQP_TEST:
        server = new(std::nothrow) Nd2ReceiveFlushQPServer;
        client = new(std::nothrow) Nd2ReceiveFlushQPClient;
        break;
    case ND2_SENDRECEIVE_CONN_CLOSED_TEST:
        server = new(std::nothrow) Nd2SendReceiveConnectorClosedServer;
        client = new(std::nothrow) Nd2SendReceiveConnectorClosedClient;
        break;
    case ND2_LARGEQP_TEST:
        server = new(std::nothrow) Nd2LargeQPDepthServer;
        client = new(std::nothrow) Nd2LargeQPDepthClient;
        break;
    case ND2_QPMAX_TEST:
        server = new(std::nothrow) Nd2QPMaxAllServer;
        client = new(std::nothrow) Nd2QPMaxAllClient;
        break;
    case ND2_MR_INVALIDBUFFER_TEST:
        server = new(std::nothrow) Nd2MRInvalidBufferServer;
        client = new(std::nothrow) Nd2MRInvalidBufferClient;
        break;
    case ND2_CONN_LISTEN_CLOSING_TEST:
        server = new(std::nothrow) Nd2ConnectListenerClosingServer;
        client = new(std::nothrow) Nd2ConnectListenerClosingClient;
        break;
    case ND2_INVALID_IP_TEST:
        server = new(std::nothrow) Nd2InvalidIPServer;
        client = new(std::nothrow) Nd2InvalidIPClient;
        break;
    case ND2_DUAL_CONECTION_TEST:
        server = new(std::nothrow) Nd2DualConnectionServer;
        client = new(std::nothrow) Nd2DualConnectionClient;
        break;
    case ND2_CONN_REJECT_TEST:
        server = new(std::nothrow) Nd2ConnRejectCloseServer;
        client = new(std::nothrow) Nd2ConnRejectClient;
        break;
    case ND2_CONN_CLOSE_TEST:
        server = new(std::nothrow) Nd2ConnRejectCloseServer;
        client = new(std::nothrow) Nd2ConnCloseClient;
        break;
    case ND2_DUAL_LISTEN_TEST:
        server = new(std::nothrow) Nd2DualListenServer;
        client = new(std::nothrow) Nd2DualListenClient;
        break;
    default:
        printf( "Bad Test ID.\n\n");
        exit( __LINE__ );
    }

    if( server == NULL || client == NULL )
    {
        printf( "Memory allocation failed.\n" );
        exit( __LINE__ );
    }

    if( bServer )
    {
        //run server
        server->RunTest( v4Server, queueDepth, nSge );
    }
    else
    {
        struct sockaddr_in v4Src;

        SIZE_T len = sizeof(v4Src);
        hr = NdResolveAddress(
            (const struct sockaddr*)&v4Server,
            sizeof(v4Server),
            (struct sockaddr*)&v4Src,
            &len );
        if( FAILED( hr ))
        {
            printf( "NdResolveAddress failed with %08x\n", hr );
            exit( __LINE__ );
        }
        if(testName ==ND2_INVALID_IP_TEST){
            v4Server.sin_addr.s_addr =0;
        }

        //run client
        client->RunTest( v4Src, v4Server, queueDepth, nSge );

    }

    delete client;
    delete server;

    hr = NdCleanup();
    if( FAILED(hr) )
    {
        printf( "NdCleanup failed with %08x\n", hr );
        exit( __LINE__ );
    }

    CloseHandle( Ov.hEvent );

    _fcloseall();
    WSACleanup();

    return 0;
}


