#include "Nd2MemoryTest.h"

//test cases for Memory region deregisteration when memory window is bound to it
void Nd2MRDeregisterServer::RunTest(
    _In_ const struct sockaddr_in& v4Src,
    _In_ DWORD queueDepth,
    _In_ DWORD nSge
    )
{
	//prep
    Nd2TestBase::Init( v4Src);
	Nd2TestBase::CreateMR();
	Nd2TestBase::RegisterDataBuffer( x_HdrLen + x_MaxXfer,ND_MR_FLAG_ALLOW_LOCAL_WRITE,ND_SUCCESS,"Register memory with local read access failed");
	Nd2TestBase::CreateCQ(nSge);
	Nd2TestBase::CreateConnector();
	Nd2TestBase::CreateQueuePair(queueDepth,nSge);
	
	//connect
	Nd2TestServerBase::CreateListen();
	Nd2TestServerBase::Listen(v4Src);
	Nd2TestServerBase::GetConnectionRequest();
	
	//accept connection			
	Nd2TestServerBase::Accept(1,1);

	//create MW and bind it to memory
	Nd2TestBase::CreateMW();
	Nd2TestBase::Bind(x_MaxXfer,ND_MR_FLAG_ALLOW_LOCAL_READ);

	//this call should fail, because there's still memory window bound to it
	HRESULT hr = m_pMr->Deregister( &m_Ov );
	LogIfErrorExit(hr,ND_DEVICE_BUSY,"Deregistering memory region with memory window bound it should get ND_DEVICE_BUSY!",__LINE__);
	
	
	//tear down
	Nd2TestBase::Shutdown();
}

void Nd2MRDeregisterClient::RunTest(
    _In_ const struct sockaddr_in& v4Src,
    _In_ const struct sockaddr_in& v4Dst,
    _In_ DWORD queueDepth,
    _In_ DWORD nSge
    )
{
	//prep    
    Nd2TestBase::Init( v4Src);
	Nd2TestBase::CreateMR();
	Nd2TestBase::RegisterDataBuffer(x_MaxXfer,ND_MR_FLAG_ALLOW_LOCAL_READ,ND_SUCCESS,"Register memory with local read access failed");
	Nd2TestBase::CreateCQ(nSge,ND_SUCCESS);
	Nd2TestBase::CreateConnector();
	Nd2TestBase::CreateQueuePair(queueDepth,nSge);
	
	//connect
	Nd2TestClientBase::Connect(v4Src,v4Dst,1,1);
	Nd2TestClientBase::CompleteConnect();
	
	//sleep 5 seconds to let the other end deregister memory
	Sleep(5*1000);
	
    Nd2TestBase::Shutdown();
}