
param(
    [string[]] $ndtests = (
        "ndcat",
        "ndconn",
        "ndcq",
        "ndping",
        "ndrping",
        "ndpingpong",
        "ndrpingpong",
        "ndmrrate",
        "ndmrlat",
        "ndmpic",
        "ndmw",
        "ndmval"
    ),
    [string[]] $mpitests = (
        "mpi_allreduce", 
        "mpi_alltoall", 
        "mpi_alltoallv",
        "mpi_bcast",
        "mpi_intercomm_create1",
        "mpi_isend_perf",
        "mpi_reduce",
        "mpi_send_flood",
        "accfence1",
        "async",
        "coll2",
        "coll13",
        "putpscw1",
        "scattern",
        "sendmany",
        "sendrecv1",
        "transpose1"
    ),
    [string[]] $nd2tests = (
        "Nd2InvalidRead",
        "Nd2InvalidWrite",
        "Nd2LargePrivateData",
        "Nd2MRDeregister",
        "Nd2OverRead",
        "Nd2OverWrite",
        "Nd2WriteViolation",
        "Nd2SendNoReceive",
        "Nd2ReceiveFlushQP",
        "Nd2SendReceiveConnClosed",
        "Nd2LargeQPDepth",
        "Nd2QpMax",
        "Nd2MRInvalidBuffer",
        "Nd2ConnListenClosing",
        "Nd2InvalidIP",
        "Nd2DualListen",
        "Nd2DualConnection",
        "Nd2ConnReject",
        "Nd2ConnClose"
    ),
    $run32=$false,
    $run64=$true,
    [string]$Version="1",
    $runnd=$true,
    $runmpi=$true,
    [string]$TestNodeGroup="ComputeNodes"
    
)

$ErrorActionPreference = "SilentlyContinue"

$output = @()
$output += Set-Location ..
$env:ndlogo_root = (Get-Location).Path

$logoStartTime = Get-Date
$resultsDirName = "Results-$($logoStartTime.ToString('yyyyMMdd-HHmmss'))"
$resultsdir = mkdir "$($env:ndlogo_root)\$resultsdirname"

"`tSaving results to $($resultsdir.FullName)"
$env:ndlogo_logfile = "$($resultsdir.FullName)\ndlogo_tests.log"

$output += . $env:ndlogo_root\scripts\load-guifunctions.ps1 

$output += Get-PSSnapin -Name Microsoft.Hpc # | Out-Null

if (-not $?)
{
    $output += Add-PSSnapin Microsoft.Hpc # | Out-Null # load HPC Powershell cmdlets
}

if (-not $?) 
{
    $errormsg = "Failed to load HPC Powershell Snapins."
    $output += show-errordialog -errormsg $errormsg
    $output >> $env:ndlogo_logfile ; exit 255
}
else
{
    $output += log "`tHPC Powershell cmdlets loaded."
}

$output += . $env:ndlogo_root\scripts\load-ndlogofunctions.ps1
$output += log "Running nd tests with -run32:$run32 -run64 $run64 -runnd $runnd -runmpi $runmpi -TestNodeGroup:$TestNodeGroup"

$output += . {
    net share /d /y ndlogo
    net share ndlogo="$($resultsdir.FullName)" /grant:"everyone,full"
} # >> $env:ndlogo_logfile

if (-not $?) 
{
    $errormsg = `
"Failed to create results share.
(Did you run this script with administrative priviliges?)"

    $output += show-errordialog -errormsg $errormsg
    $output >> $env:ndlogo_logfile ; exit 255
}

$output += . { 
    net share /d /y ndlogotests64
    net share ndlogotests64="$($env:ndlogo_root)\bin\amd64" /grant:"everyone,full"
} # >> $env:ndlogo_logfile 

if (-not $?) 
{
    $errormsg = `
"Failed to create test binaries share.
(Did you run this script with administrative priviliges?)"

    $output += show-errordialog -errormsg $errormsg
    $output >> $env:ndlogo_logfile ; exit 255
}

$env:ndlogo_testshare64 = "\\$(hostname)\ndlogotests64"
$output += log "`tNDLogo64 Tests Share: $($env:ndlogo_testshare64)"
    
if($run32){
  $output += . { 
    net share /d /y ndlogotests32
    net share ndlogotests32="$($env:ndlogo_root)\bin\i386" /grant:"everyone,full"
  }
  if (-not $?) 
  {
    $errormsg = `
"Failed to create test binaries share.
(Did you run this script with administrative priviliges?)"

    $output += show-errordialog -errormsg $errormsg
    $output >> $env:ndlogo_logfile ; exit 255
  }
  $env:ndlogo_testshare32 = "\\$(hostname)\ndlogotests32"
  $output += log "`tNDLogo32 Tests Share: $($env:ndlogo_testshare32)"
}


$env:ndlogo_share = "\\$(hostname)\ndlogo"
$output += log "`tNDLogo Share:  $($env:ndlogo_share)"

$output += . { 
    net share /d /y ndlogoscripts
    net share ndlogoscripts="$($env:ndlogo_root)\scripts" /grant:"everyone,full"
} # >> $env:ndlogo_logfile 

if (-not $?) 
{
    $errormsg = `
"Failed to create test scripts share.
(Did you run this script with administrative priviliges?)"

    $output += show-errordialog -errormsg $errormsg
    $output >> $env:ndlogo_logfile ; exit 255
}

$env:ndlogoscripts = "\\$(hostname)\ndlogoscripts"
$output += log "`tNDLogo64 Scripts Share: $($env:ndlogoscripts)"


if (-not (( Get-HpcNetworkTopology ) -match "Application"))
{
    $errormsg = 
"You haven't configured an Application network.
Please check your network topology in the Cluster Manager.
You must configure the 'Private / Application' or the 
'Enterprise / Private / Application' topology, with your 
NetworkDirect devices on the Application network."

    $output += show-errordialog -errormsg $errormsg
    $output >> $env:ndlogo_logfile ; exit 255
}

$headnodend = Get-HpcNetworkInterface -Type Application

if (-not $headnodend) 
{
    $errormsg = 
"Your Application network seems to be misconfigured.
Your headnode does not have a NIC identified as 'Application'.
Your headnode must have a NetworkDirect device installed on 
its Application network interface."

    $output += show-errordialog -errormsg $errormsg
    $output >> $env:ndlogo_logfile ; exit 255
}

$output += .$env:ndlogo_testshare64\ndcat.exe $headnodend.IpAddress.Trim() # >> $env:ndlogo_logfile
if (-not $?) 
{
    $errormsg = `
"Your Application network seems to be misconfigured.
NetworkDirect does not appear to be enabled 
on your Application network interface."

    $output += show-errordialog -errormsg $errormsg
    $output >> $env:ndlogo_logfile ; exit 255
}
else
{
$output += log `
"`tApplication network configured: $($headnodend.IpAddress)/$($headnodend.SubnetMask)
`tNetworkDirect-enabled device detected on Application network: $($headnodend.Name)"
}

$output += log "`tTaking headnode offline for NetworkDirect functional tests..."
$output += Get-HpcNode -GroupName HeadNodes | Set-HpcNodeState -State Offline 

$onlinenodes = (Get-HpcNode -groupname "$TestNodeGroup" -state online).count
if ($onlinenodes -lt 2)
{
    $errormsg = 
"Your cluster must contain at least 2 online nodes in nodegroup $TestNodeGroup to run
the NetworkDirect logo suite."

    $output += show-errordialog -errormsg $errormsg
    $output >> $env:ndlogo_logfile ; exit 255
}
if (($onlinenodes -lt 8) -and ($runmpi -eq $true))
{
    $errormsg = 
"Your cluster must contain at least 8 online nodes in nodegroup $TestNodeGroup to run
the NetworkDirect logo suite with mpi tests."

    $output += show-errordialog -errormsg $errormsg
    $output >> $env:ndlogo_logfile ; exit 255
}


$output += Get-HpcGroup -Name NDLogo # >> $env:ndlogo_logfile

if ($?) { $groupname = "NDLogo" } else { $groupname = $TestNodeGroup}

$output += log "`tUsing group $groupname for NetworkDirect functional tests..."
$output >> $env:ndlogo_logfile


if($runnd -eq $true){
    $output = @()
    if ($Version.contains("1"))
    {
      foreach ($ndtest in $ndtests) 
      {
          $output += log "`tRunning test: $ndtest ..."
          run-ndtest -test $ndtest
          $result = check-ndtest -test $ndtest `
                          -outdir $env:ndlogo_share\$ndtest `
                          -summaryfile $env:ndlogo_share\results_summary.txt
          $output += log "`t`t$result"
      }
    }

    if ($Version.contains("2"))
    {
      $output += log "Running nd2 test cases"
      run-nd2test -tests $nd2tests
    }

    $output >>  $env:ndlogo_logfile
}
$output = @()

$output += Get-HpcNode -GroupName Headnodes | Set-HpcNodeState -State Online

$output += Set-HpcClusterProperty -Environment "MPICH_DISABLE_SOCK=1","MPICH_DISABLE_ND=0"

if($runmpi -eq $true)
{
    foreach ($mpitest in $mpitests)
    {
        $output += log "`tRunning test: $mpitest ..."
    
        $job, $output2 = run-ndmpitest -testname $mpitest -groupname $groupname -ndlogotests $env:ndlogo_testshare64
        if($run32)
        {
          $job, $output2 = run-ndmpitest -testname $mpitest -groupname $groupname -ndlogotests $env:ndlogo_testshare32
        }
        $output += $output2
        $result = check-mpitest -test $mpitest `
                        -job $job `
                        -outdir $env:ndlogo_share\$mpitest `
                        -summaryfile $env:ndlogo_share\results_summary.txt
        $output += log "`t`t$result"
    }
}
$output >> $env:ndlogo_logfile
$output = @()

#process results

$results = @()
$results += "Summary of results ($((get-item $env:ndlogo_share\results_summary.txt).FullName)):"
$results += (gc $env:ndlogo_share\results_summary.txt)

$resultszip = zip-ndresultsfolder( $resultsdir )


$message = 
"
The NetworkDirect Logo test suite has finished.

Your results for submission have been placed in
$resultszip"

$results | out-host

$output += show-alertdialog -message $message

$output >> $env:ndlogo_logfile
